#!/bin/bash
#
#       82 - lnadmin user, backup scripts and administration panel setup
#

date=`date +"%d/%m/%Y %H:%M"`
echo -en "\n\n****************************************************\n"
echo -en "[$date] lnadmin user and backup scripts setup\n"
echo "****************************************************"

# User creation
echo "Creant l'usuari lnadmin"
res_value=0
res=$(getent passwd lnadmin || useradd -m -r -p $1 -s /bin/bash --service files lnadmin 2>&1)
if [ ! $? -eq 0 ];then
	echo -en "Error:: No s'ha pogut crear l'usuari lnadmin; $res"
	res_value=1
else
	echo -en "Success:: Usuari lnadmin creat\n"
fi

# Only proceed if lnadmin user is properly created in the system, otherwise abort for security reasons
if [ $res_value -eq 0 ]; then
	echo -en "Establint permisos d'escriptura a la unitat S: "
	res=$(setfacl -m user:lnadmin:rwx /srv/exports/S 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Problemes establint ACLs a /srv/exports/S ; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: Permisos d'escriptura sobre la unitat S atorgats satisfactòriament\n"
	fi
	echo -en "Establint permisos d'escriptura a la unitat G "
	res=$(setfacl -m user:lnadmin:rwx /srv/exports/G 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Problemes establint ACLs a /srv/exports/G ; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: Permisos d'escriptura sobre la unitat G atorgats satisfactòriament\n"
	fi
        echo -en "Establint permisos d'escriptura a la unitat G per a grup gestio "
        res=$(setfacl -m g:Gestio:rwx /srv/exports/G 2>&1)
        if [ ! $? -eq 0 ];then
                echo -en "Error:: Problemes establint ACLs de gestio a /srv/exports/G ; $res\n"
        else
                echo -en "Ok\n"
                echo -en "Success:: Permisos d'escriptura sobre la unitat G atorgats satisfactòriament\n"
        fi
	
### Establint ACLS a la paperera de reciclatge G	
	echo -en "Establint permisos d'escriptura a la paperera  G per usuari lnadmin "
        res=$(setfacl -m user:lnadmin:rwx /srv/exports/Paperera/Paperera_Unitat_G 2>&1)
        if [ ! $? -eq 0 ];then
                echo -en "Error:: Problemes establint ACLs a /srv/exports/Paperera/Paperera_Unitat_G ; $res\n"
        else
                echo -en "Ok\n"
                echo -en "Success:: Permisos d'escriptura sobre la paperera G atorgats satisfactòriament\n"
        fi
	
	res=$(setfacl -m g:Gestio:rwx /srv/exports/Paperera/Paperera_Unitat_G 2>&1)
        if [ ! $? -eq 0 ];then
                echo -en "Error:: Problemes establint ACLs de gestio a /srv/exports/Paperera/Paperera_Unitat_G ; $res\n"
        else
                echo -en "Ok\n"
                echo -en "Success:: Permisos d'escriptura sobre la Paperera G atorgats satisfactòriament\n"
        fi
	

	# Sudoers setup for yast2 graphical module access
	echo -en "Atorgant permisos d'administració de varis mòduls de yast a lnadmin \n"
	if [ -e /etc/sudoers ];then
		echo -en "Fent còpia de seguretat del sudoers:"
		res=$(cp /etc/sudoers $DIR_LINKAT/backup/sudoers.orig 2>&1)
		if [ ! $? -eq 0 ];then
			echo -en "\n\tError:: S'ha produït un error al fer el backup del fitxer sudoers; $res\n"
		else
			echo -en "Ok\n"
		fi
	fi 
	res=$(cp -f $DIR_LINKAT/data/sudoers.Linkat /etc/sudoers 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Problemes escrivint el fitxer sudoers amb permisos per lnadmin; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: Canvis aplicats al fitxer /etc/sudoers\n"
	fi
	
	# Policykit permissions setup
	echo -en "Establint permisos de policykit per la impressora "
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.server-settings 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a canviar server-settings; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar la configuració del CUPS\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.printeraddremove 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a afegir i eliminar impressores; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a afegir i eliminar impressores\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.printer-set-default 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a canviar la impressora per defecte; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar la impressora per defecte\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.printer-remote-edit 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a modificar una impressora remota; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a modificar una impressora remota\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.printer-local-edit 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a canviar les impressores locals; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar les impressores locals\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.printer-enable 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a habilitar impressores; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a habilitar i deshabilitar impressores\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.opensuse.cupspkhelper.mechanism.class-edit 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a editar classes d'impressió; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar classes d'impressió\n"
	fi
	
	# Clock applet permissions
	echo -en "Autoritzant lnadmin a canviar l'hora i fus horari del sistema"
	res=$(polkit-auth --user lnadmin --grant org.gnome.clockapplet.mechanism.settimezone 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a canviar el fus horari del sistema; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar el fus horari del sistema\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.gnome.clockapplet.mechanism.settime 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin a canviar l'hora del sistema; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar l'hora del sistema (settime)\n"
	fi
	res=$(polkit-auth --user lnadmin --grant org.gnome.clockapplet.mechanism.configurehwclock 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin per canviar el rellotge del sistema; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat a canviar l'hora del sistema (hwclock)\n"
	fi
	
	# Package updates
	echo -en "Autoritzant lnadmin a actualitzar el sistema: "
	res=$(polkit-auth --user lnadmin --grant org.freedesktop.packagekit.system-update 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar lnadmin per actualitzar el sistema; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat (system-update)\n"
	fi
	echo -en "Autoritzant lnadmin a confiar en claus de firma: "
	res=$(polkit-auth --user lnadmin --grant org.freedesktop.packagekit.system-trust-signing-key 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar a lnadmin; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat (system-trust-signing-key)\n"
	fi
	echo -en "Autoritzant lnadmin a configurar el proxy de xarxa: "
	res=$(polkit-auth --user lnadmin --grant org.freedesktop.packagekit.system-network-proxy-configure 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar a lnadmin; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat (system-network-proxy-configure)\n"
	fi
	echo -en "Autoritzant lnadmin a acceptar llicències d'un nou paquet: "
	res=$(polkit-auth --user lnadmin --grant org.freedesktop.packagekit.package-eula-accept 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut autoritzar a lnadmin; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: lnadmin autoritzat (package-eula-accept)\n"
	fi
	
	# lnadmin session autostart commands
	echo -en "Configuració de comandes d'arrencada de l'usuari lnadmin: \n"
	res=$(mkdir -p /home/lnadmin/.config/autostart/ && chown lnadmin:users /home/lnadmin/.config/ && chown lnadmin:users /home/lnadmin/.config/autostart/ 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut crear correctament el directori autostart de lnadmin; $res\n"
	else
		# xhost + for lnadmin X session
		echo -en "Autoritzant accés local a l'escriptori de lnadmin: "
		res=$(cp $DIR_LINKAT/data/xhost.desktop /home/lnadmin/.config/autostart/ && chown lnadmin:users /home/lnadmin/.config/autostart/xhost.desktop && chmod 644 /home/lnadmin/.config/autostart/xhost.desktop 2>&1)
		if [ ! $? -eq 0 ];then
			echo -en "Error:: No s'ha pogut autoritzar l'accés de root a les X de lnadmin; $res\n"
		else
			echo -en "Ok\n"
			echo -en "Success:: Autoritzat accés local a l'escriptori de lnadmin\n"
		fi
	fi

	# Allow lnadmin to read the stored LDAP cn=Administrator password
	echo -en "Permetent que l'usuari lnadmin llegeixi la contrasenya d'administrador de l'LDAP: \n"
	res=$(setfacl -m user:lnadmin:r-- $DIR_LINKAT/data/.passwd 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut establir l'ACL a $DIR_LINKAT/data/.passwd; $res\n"
	else
		res=$(setfacl -m user:lnadmin:r-x $DIR_LINKAT/data 2>&1)
		if [ ! $? -eq 0 ];then
			echo -en "Error:: No s'ha pogut establir l'ACL a $DIR_LINKAT/data; $res\n"
		else
			res=$(setfacl -m user:lnadmin:r-x $DIR_LINKAT 2>&1)
			if [ ! $? -eq 0 ];then
				echo -en "Error:: No s'ha pogut establir l'ACL a $DIR_LINKAT; $res\n"
			else
				echo -en "Success:: Autoritzat accés de lectura a $DIR_LINKAT/data/.passwd\n"
			fi
		fi
	fi

	#Allow lnadmin read/write in backup media mount files
	echo -en "Permetent que l'usuari lnadmin sigui el propietari de /etc/remmount.d: \n"
	res=$(chown lnadmin.users /etc/remmount.d 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Error al cambiar el propietari a /etc/remmount.d; $res\n"
	else
		echo -en "Ok!:: Propietari establert correctament; $res\n"
	fi	

	# Desktop icons setup
	echo -en "Instal·lant icones d'administració del servidor: "
	res=$(mkdir -p /home/lnadmin/Desktop && chown lnadmin:users /home/lnadmin/Desktop && cd /home/lnadmin/Desktop && tar -zxvf $DIR_LINKAT/data/eines_lnadmin_servidor.tar.gz && chown -R root:root /home/lnadmin/Desktop/Eines\ d\'administració 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'han pogut extreure les icones d'administració del servidor; $res\n"
	else
		echo -en "Ok\n"
		res=$(chmod 755 /home/lnadmin/Desktop/Eines\ d\'administració/*.desktop 2>&1)
	     	if [ ! $? -eq 0 ];then
			echo -en "Error:: No s'han pogut canviar els permisos de les icones d'administració del servidor; $res\n"
		else
			echo -en "Success:: Icones de les eines d'administració del servidor instal·lades correctament\n"
			echo -en "Creant un enllaç simbòlic a l'escriptori cap al fitxer de contrasenya LDAP del servidor: "
			res=$(ln -s $DIR_LINKAT/data/.passwd /home/lnadmin/Desktop/Eines\ d\'administració/Contrasenya\ del\ servidor\ LDAP 2>&1)
			if [ ! $? -eq 0 ];then
				echo -en "Error:: No s'ha pogut crear el symlink contra $DIR_LINKAT/data/.passwd; $res\n"
			else
				echo -en "Ok\n"
			fi
		res=$(ln -s /etc/remmount.d/ /home/lnadmin/Desktop/Eines\ d\'administració/recursos_REMMOUNT 2>&1)
		fi
	fi
fi

