#!/bin/bash
#
# Zenity GUI to setup network parameters giving default values.
#

ZENITY=$(which zenity)
#DEV_FILE=$DIR_NET/data/net_device

IP="192.168.0.240"
NET="255.255.255.0"
GATEWAY="192.168.0.1"


check_ip()
{
	echo "$1" | grep -E '^(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])$' > /dev/null 2>&1
	return $?
}

LIST_DEV=`ip link sh | grep ^[0-9] | grep -v " lo" | cut -d":" -f 2 | tr -d " "`
LIST_DEVS=$(for i in $LIST_DEV; do echo FALSE $i; done)
DEV=""

while [ "$DEV" == "" ];
do
        DEV=$(echo $LIST_DEV | zenity --list \
         --title "Seleccioneu el dispositiu de xarxa" \
         --text "Seleccioneu el dispositiu de xarxa on s'ofereixen els serveis." \
         --radiolist --column "Seleccioneu" \
         --column "Dispositiu"  $LIST_DEVS )
done


FI=0
while [ $FI -eq 0 ] ; do
	FI=1
	INFO=$($ZENITY --title="Configuració IP" --height="250" --width="500" --list --text="Configureu els paràmetres de la xarxa:" --column="IP" --column="NETMASK" --column="GATEWAY" --column="DNS" ${IP} ${NET} ${GATEWAY} ${DNS1} --editable --print-column="ALL" --separator=":")
	if [ $? -ne 0 ] ; then
		FI=0
		$ZENITY --error --text="Si us plau accepteu la configuració." --timeout=20
	else
		if [ -z $INFO ] ; then
			FI=0
			$ZENITY --error --text="Si us plau seleccioneu una línia." --timeout=20
		else
			NEW_IP=$(echo $INFO | cut -d ":" -f 1)
			check_ip $NEW_IP
			if [ $? -ne 0 ]; then
				FI=0
				$ZENITY --error --text="L'adreça IP $NEW_IP no és correcte, si us plau introduïu-la de nou." --timeout=20
			else
				NEW_NET=$(echo $INFO | cut -d ":" -f 2)
				check_ip $NEW_NET
				if [ $? -ne 0 ]; then
					FI=0
					$ZENITY --error --text="La màscara de xarxa $NEW_NET no és correcte, si us plau introduïu-la de nou." --timeout=20
					#TODO: Check if netmask is valid (if have the form 1*0* in binary notation).
				else
					NEW_GATEWAY=$(echo $INFO | cut -d ":" -f 3)
					check_ip $NEW_GATEWAY
					if [ $? -ne 0 ]; then
						FI=0
						$ZENITY --error --text="La porta d'enllaç $NEW_GATEWAY no és correcte, si us plau introduïu-la de nou." --timeout=20
					else
						NEW_DNS=$(echo $INFO | cut -d ":" -f 4)
						check_ip $NEW_DNS
						if [ $? -ne 0 ]; then
							FI=0
							$ZENITY --error --text="El servidor DNS $NEW_DNS no és correcte, si us plau introduïu-lo de nou." --timeout=20
						fi
					fi
				fi
			fi
		fi
	fi
done


echo "export NEW_IP=$NEW_IP" >> $DIR_LINKAT/.entornvarlinkat

echo "export NEW_GATEWAY=$NEW_GATEWAY" >> $DIR_LINKAT/.entornvarlinkat

echo $DEV > $DIR_NET/data/net_device

cat <<EOF > $DIR_NET/data/$DEV.cfg
auto $DEV
iface $DEV inet static
address $NEW_IP
netmask $NEW_NET
gateway $NEW_GATEWAY
dns-nameserver $NEW_DNS 127.0.0.1
EOF

