#!/bin/bash
echo -en  "\n*** 05-cert-ssl \n"
. /etc/Linkat/linkat-server/serveis/LDAP/install/config-ldap

err=0

killall update-manager update-notifier

sudo apt-get -y install gnutls-bin ssl-cert

if [ ! $? -eq 0 ]; then
	echo -en "Error:: No s'ha pogut instal·lar els paquet necessaris del certificat"
	err=1
else
	echo -en "Succeed: Paquets de certificació instal·lats"
fi

sudo sh -c "certtool --generate-privkey > /etc/ssl/private/cakey.pem"

if [ ! $? -eq 0 ]; then
        echo -en "Error:: No s'ha pogut generar cakey.pem"
	err=1
else
        echo -en "Succeed: Cakey generat"
fi


### Modificar les variables ###
cat $DATA/ca.info.linkat | \
	sed s/__DOMINI__/$DOMINI/g \
	> $DATA/ca.info
if [ ! $? -eq 0 ]; then
        echo -en "Error:: No s'ha pogut modificar les variables ca.info"
        err=1
else
        echo -en "Succeed: Variables modificats ca.info"
fi


cat  $DATA/servidor.info.linkat | \
	sed s/__NOM__/$NOM/g | \
	sed s/__DOMINI__/$DOMINI/g \
	> $DATA/servidor.info
if [ ! $? -eq 0 ]; then
        echo -en "Error:: No s'ha pogut modificar les variables servidor.info"
        err=1
else
        echo -en "Succeed: Variables modificats servidor.info"
fi


### Generar certificats i claus ###	
sudo certtool --generate-self-signed \
--load-privkey /etc/ssl/private/cakey.pem \
--template $DATA/ca.info \
--outfile /etc/ssl/certs/cacert.pem

if [ ! $? -eq 0 ]; then
        echo -en "Error: No s'ha pogut generar cacert.pem"
        err=1
else
        echo -en "cacert generat"
fi

sudo certtool --generate-privkey \
--bits 2048 \
--outfile /etc/ssl/private/servidor_slapd_key.pem

if [ ! $? -eq 0 ]; then
        echo -en "Error: No s'ha pogut generar servidor_slapd_key.pem"
        err=1 
else
        echo -en "servidor_slapd_key.pem generat"
fi

sudo certtool --generate-certificate \
--load-privkey /etc/ssl/private/servidor_slapd_key.pem \
--load-ca-certificate /etc/ssl/certs/cacert.pem \
--load-ca-privkey /etc/ssl/private/cakey.pem \
--template $DATA/servidor.info \
--outfile /etc/ssl/certs/servidor_slapd_cert.pem

if [ ! $? -eq 0 ]; then
        echo -en "Error: No s'ha pogut generar servidor_slapd_cert.pem"
        err=1
else
        echo -en "servidor_slapd_cert.pem generat"
fi

sudo ldapmodify -Y EXTERNAL -H ldapi:/// -f $DATA/certinfo.ldif

if [ ! $? -eq 0 ]; then
        echo -en "Error: No s'ha pogut modificar LDAP"
        err=1
else
        echo -en "Modificat LDAP"
fi

sudo adduser openldap ssl-cert
sudo chgrp ssl-cert /etc/ssl/private/servidor_slapd_key.pem
sudo chmod g+r /etc/ssl/private/servidor_slapd_key.pem
sudo chmod o-r /etc/ssl/private/servidor_slapd_key.pem

sudo service slapd restart

### Esborrant temporals
rm $DATA/ca.info $DATA/servidor.info

if [ ! $err -eq 0 ]; then
        echo "Alguna cosa no ha anat correctament!" >> $LDAPLOG
else
        echo "Tot correcte" >> $LDAPLOG
fi
