#!/bin/bash
#
#       82 - lnadmin user, backup scripts and administration panel setup
#

. /etc/Linkat/linkat-server/serveis/LNADMIN/install/config-lnadmin


date=`date +"%d/%m/%Y %H:%M"`
echo -en "\n\n****************************************************\n"
echo -en "[$date] lnadmin user and backup scripts setup\n"
echo "****************************************************\n"

# User creation
echo "Creant l'usuari lnadmin"
res_value=0
getent passwd lnadmin 
if [ $? -eq 0 ]; then
	userdel lnadmin
	echo -en "Esborrant l'usuari lnadmin\n"
	if [ $? -eq 0 ];then
        	echo -en "Success:: S'ha eliminat l'usuari lnadmin.\n"
	else
        	echo -en "Error:: No s'ha pogut eliminar Usuari lnadmin.\n"
		res_value=1
	fi
fi

adduser --system --home /home/lnadmin --shel /bin/bash lnadmin 
if [ ! $? -eq 0 ];then
	echo -en "Error:: No s'ha pogut crear l'usuari lnadmin\n"
	res_value=1
else
	echo -en "Success:: Usuari lnadmin creat\n"
fi

## Modificant variables de passwd-lnadmin.sh
cat $DATA/02-passwd-lnadmin.sh.linkat | \
	sed s/__LNADMIN_PASS__/$PASS_LNADMIN/g \
	 > $INSTALL/02-passwd-lnadmin.sh

        if [ $? -eq 0 ];then
                echo -en "Success:: S'han modificat les variables de passwd-lnadmin.sh.\n"
                chmod 750 $INSTALL/02-passwd-lnadmin.sh

	else
		echo -en "Error:: No s'han modificat les variables de passwd-lnadmin.sh.\n"
                res_value=1
        fi


# Only proceed if lnadmin user is properly created in the system, otherwise abort for security reasons
if [ $res_value -eq 0 ]; then
	echo -en "Establint permisos d'escriptura a la unitat S: "
	res=$(setfacl -R -m default:user:lnadmin:rwx -m user:lnadmin:rwx /srv/exports/S 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Problemes establint ACLs a /srv/exports/S ; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: Permisos d'escriptura sobre la unitat S atorgats satisfactòriament\n"
	fi
	echo -en "Establint permisos d'escriptura a la unitat G "
	res=$(setfacl -m default:user:lnadmin:rwx -m user:lnadmin:rwx /srv/exports/G 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Problemes establint ACLs a /srv/exports/G ; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: Permisos d'escriptura sobre la unitat G atorgats satisfactòriament\n"
	fi
        echo -en "Establint permisos d'escriptura a la unitat G per a grup gestio "
        res=$(setfacl -m default:user:lnadmin:rwx -m user:lnadmin:rwx /srv/exports/M 2>&1)
        if [ ! $? -eq 0 ];then
                echo -en "Error:: Problemes establint ACLs de gestio a /srv/exports/M ; $res\n"
        else
                echo -en "Ok\n"
                echo -en "Success:: Permisos d'escriptura sobre la unitat M atorgats satisfactòriament\n"
        fi
        res=$(setfacl -m default:user:lnadmin:rwx -m user:lnadmin:rwx /srv/exports/P 2>&1)
        if [ ! $? -eq 0 ];then
                echo -en "Error:: Problemes establint ACLs de gestio a /srv/exports/P ; $res\n"
        else
                echo -en "Ok\n"
                echo -en "Success:: Permisos d'escriptura sobre la unitat P atorgats satisfactòriament\n"
        fi

	
	# Sudoers setup 
	echo -en "Atorgant permisos d'administració a lnadmin \n"
	if [ -e /etc/sudoers ];then
		echo -en "Fent còpia de seguretat del sudoers:"
		res=$(cp /etc/sudoers $DIR_LNADMIN/backup/sudoers.orig 2>&1)
		if [ ! $? -eq 0 ];then
			echo -en "\n\tError:: S'ha produït un error al fer el backup del fitxer sudoers; $res\n"
		else
			echo -en "Ok\n"
		fi
	fi 
	res=$(cp -f $DIR_LNADMIN/data/sudoers.linkat /etc/sudoers 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Problemes escrivint el fitxer sudoers amb permisos per lnadmin; $res\n"
	else
		echo -en "Ok\n"
		echo -en "Success:: Canvis aplicats al fitxer /etc/sudoers\n"
	fi

	# Allow lnadmin to add and remove printers
	cp $DATA/linkat.desktop.pkla /var/lib/polkit-1/localauthority/20-org.d/ 
        if [ $? -eq 0 ];then
                echo -en "Success:: S'ha copiat correctament linkat.desktop.pkla per lnadmin\n"
        else
                echo -en "Error:: No s'ha copiat correctament linkat.desktop.pkla per lnadmin\n"
        fi

	# Allow lnadmin to read the stored LDAP cn=Administrator password
	echo -en "Permetent que l'usuari lnadmin llegeixi la contrasenya d'administrador de l'LDAP: \n"
	res=$(setfacl -m user:lnadmin:r-- /etc/ldap.secret 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'ha pogut establir l'ACL a /etc/ldap.secret; $res\n"
	fi

	# Allow lnadmin read/write in backup media mount files
	echo -en "Permetent que l'usuari lnadmin sigui el propietari de /etc/remmount.d: \n"
	res=$(chown lnadmin /etc/remmount.d 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: Error al cambiar el propietari a /etc/remmount.d; $res\n"
	else
		echo -en "Ok!:: Propietari establert correctament; $res\n"
	fi	

	# Config /home/lnadmin
	echo -en "Configurant l'escriptori de l'usuari lnadmin"
	res=$(mkdir -p /home/lnadmin/.config  && cd /home/lnadmin/.config && cp /etc/Linkat/linkat-server/serveis/LNADMIN/data/user-dirs.* . && chown -R lnadmin /home/lnadmin/.config)
	
	# Desktop icons setup
	echo -en "Instal·lant icones d'administració del servidor: "
	res=$(mkdir -p /home/lnadmin/Escriptori && chown lnadmin /home/lnadmin/Escriptori && cd /home/lnadmin/Escriptori && tar -zxvf $DIR_LNADMIN/data/eines_lnadmin_servidor.tar.gz && chown -R root:root /home/lnadmin/Escriptori/Eines\ d\'administració 2>&1)
	if [ ! $? -eq 0 ];then
		echo -en "Error:: No s'han pogut extreure les icones d'administració del servidor; $res\n"
	else
		echo -en "Ok\n"
		res=$(chmod 755 /home/lnadmin/Escriptori/Eines\ d\'administració/*.desktop 2>&1)
	     	if [ ! $? -eq 0 ];then
			echo -en "Error:: No s'han pogut canviar els permisos de les icones d'administració del servidor; $res\n"
		else
			echo -en "Success:: Icones de les eines d'administració del servidor instal·lades correctament\n"
		#	echo -en "Creant un enllaç simbòlic a l'escriptori cap al fitxer de contrasenya LDAP del servidor: "
		#	res=$(ln -s /etc/ldap.secret /home/lnadmin/Escriptori/Eines\ d\'administració/Contrasenya\ del\ servidor\ LDAP 2>&1)
		#	if [ ! $? -eq 0 ];then
		#		echo -en "Error:: No s'ha pogut crear el symlink contra /etc/ldap-secret; $res\n"
		#	else
		#		echo -en "Ok\n"
		#	fi
	#	res=$(ln -s /etc/remmount.d/ /home/lnadmin/Escriptori/Eines\ d\'administració/recursos_REMMOUNT 2>&1)
		fi
	fi
fi

chown -R lnadmin.nogroup /home/lnadmin
       if [ $? -eq 0 ];then
                echo -en "Success:: S'han modificat els permisos del /home per lnadmin\n"
       else
                echo -en "Error:: No s'han modificat els permisos del /home per lnadmin\n"
       fi

chown -R root.root /home/lnadmin/Escriptori/Eines\ d\'administració/
       if [ $? -eq 0 ];then
                echo -en "Success:: S'han modificat els permisos d'Eines a lnadmin\n"
       else
                echo -en "Error:: No s'han modificat els permisos d'Eines a lnadmin\n"
       fi
